import { useRef, useEffect, useMemo } from 'react';
import { 
  select, 
  hierarchy, 
  forceSimulation, 
  forceCenter, 
  forceManyBody,
  pointer,
  forceX,
  forceY,
  forceCollide,
  forceRadial,
  forceLink
} from 'd3';

// D3 as React Component
export const D3Component = ({ data, innerWidth, innerHeight }) => {
  const gRef = useRef();
  
  const [nodeData, linkData] = useMemo(() => {
    const root = hierarchy(data);
    const nodeData = root.descendants();
    const linkData = root.links();
    return [nodeData, linkData];
  }, [data])
  
  useEffect(() => {
    const g = select(gRef.current);
    
    const simulation = forceSimulation(nodeData)
    	// simulation start & end settings
    	.alphaTarget(0.001)
    	.alphaMin(0.002)
    	// set forces
    	.force("center", forceCenter(innerWidth/2, innerHeight/2))
    	.force("charge", forceManyBody().strength(-30)) // positive: nodes attract each other, negative: nodes repel each other
    	.force("collid", forceCollide(30)) // minimum distant of nodes from each other
	    .force('link', forceLink(linkData).strength(0.5)) // attraction between nodes connected with links
    	.on("tick", () => {
        
        // render nodes
        g.selectAll(".node").data(nodeData)
        	.join("circle")
        		.classed("node", true)
        		.attr("r", 4)
        		.attr("cx", node => node.x)
        		.attr("cy", node => node.y);
        
        // render links
        g.selectAll(".link").data(linkData)
        	.join("line")
        		.classed("link", true)
        		.attr("stroke", "black")
        		.attr("x1", link => link.source.x)
        		.attr("y1", link => link.source.y)
        		.attr("x2", link => link.target.x)
        		.attr("y2", link => link.target.y)
      //console.log("current force", simulation.alpha());
      });
    
    // Hover Event
    const parentG = select(gRef.current.parentElement);
    parentG.on("mousemove", (e) => {
      const [x, y] = pointer(e);
      simulation
      	.force("x", forceX(x).strength(node => 0.2 + node.depth * 0.1))
      	.force("y", forceY(y).strength(node => 0.2 + node.depth * 0.1));
    });
    
    // Click Event
    parentG.on("click", (e) => {
      const [x, y] = pointer(e);
      simulation.alpha(0.5).restart()
      	.force("orbit", forceRadial(100, x, y).strength(0.8));
      
      // render a circle to show radial force
      g.selectAll(".orbit").data([null])
      	.join("circle")
      		.classed("orbit", true)
      		.attr("stroke", "green")
      		.attr("fill", "none")
      		.attr("r", 100)
      		.attr("cx", x)
      		.attr("cy", y)
    });
    
    // // centering workaround (without forceCenter())
    // parentG.attr("viewBox", [
    //   -innerWidth/2,
    //   -innerHeight/2,
    //   innerWidth,
    //   innerHeight
    // ]);
    
  }, [data]);
	
  return <g ref={gRef} />;
};
